//===========================================================================
//  $Name: arts++-1-1-a6 $
//  $Id: ArtsBgp4AggregatorAttribute.hh,v 1.5 2000/01/21 08:22:42 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSBGP4AGGREGATORATTRIBUTE_HH_
#define _ARTSBGP4AGGREGATORATTRIBUTE_HH_

extern "C" {
#include "caida_t.h"
}

#include "ArtsBgp4Prefix.hh"

//---------------------------------------------------------------------------
//  class ArtsBgp4AggregatorAttribute
//---------------------------------------------------------------------------
//!  This class abstracts a BGP4 aggregator attribute.  An aggregator
//!  attribute contains an AS number and an IP address.
//---------------------------------------------------------------------------
class ArtsBgp4AggregatorAttribute
{
public:
  //--------------------------------------------------------------------------
  //                      ArtsBgp4AggregatorAttribute()
  //..........................................................................
  //!  constructor
  //--------------------------------------------------------------------------
  ArtsBgp4AggregatorAttribute();

  //--------------------------------------------------------------------------
  // ArtsBgp4AggregatorAttribute(const ArtsBgp4AggregatorAttribute & aggAttr) 
  //..........................................................................
  //!  copy constructor
  //--------------------------------------------------------------------------
  ArtsBgp4AggregatorAttribute(const ArtsBgp4AggregatorAttribute & aggAttr);
  
  //--------------------------------------------------------------------------
  //                     ~ArtsBgp4AggregatorAttribute()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsBgp4AggregatorAttribute();

  //-------------------------------------------------------------------------
  //                           uint16_t AS() const
  //.........................................................................
  //!  Returns the AS number in the ArtsBgp4AggregatorAttribute.
  //-------------------------------------------------------------------------
  uint16_t AS() const;
  
  //-------------------------------------------------------------------------
  //                        uint16_t AS(uint16_t as)
  //.........................................................................
  //!  Sets and returns the AS number in the ArtsBgp4AggregatorAttribute.
  //-------------------------------------------------------------------------
  uint16_t AS(uint16_t as);
  
  //-------------------------------------------------------------------------
  //                     ipv4addr_t & IpAddr() const
  //.........................................................................
  //!  Returns the IP address in the ArtsBgp4AggregatorAttribute.
  //-------------------------------------------------------------------------
  const ipv4addr_t & IpAddr() const;

  //-------------------------------------------------------------------------
  //          const ipv4addr_t & IpAddr(const ipv4addr_t & ipAddr)
  //.........................................................................
  //!  Sets and returns the IP address in the ArtsBgp4AggregatorAttribute.
  //-------------------------------------------------------------------------
  const ipv4addr_t & IpAddr(const ipv4addr_t & ipAddr);
  
  //-------------------------------------------------------------------------
  //       ArtsBgp4AggregatorAttribute &                      
  //       operator = (const ArtsBgp4AggregatorAttribute & aggregator)
  //.........................................................................
  //!  Overloaded '=' operator for assigning an ArtsBgp4AggregatorAttribute
  //!  to another.
  //-------------------------------------------------------------------------
  ArtsBgp4AggregatorAttribute &
  operator = (const ArtsBgp4AggregatorAttribute & aggregator);

  //--------------------------------------------------------------------------
  //            istream & read(istream & is, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4AggregatorAttribute from an
  //!  istream.  Returns the istream.
  //--------------------------------------------------------------------------
  istream & read(istream & is, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4AggregatorAttribute from a file
  //!  descriptor.  Returns the number of bytes read.
  //--------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //        ostream & write(ostream & os, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4AggregatorAttribute to an
  //!  ostream.  Returns the ostream.
  //--------------------------------------------------------------------------
  ostream & write(ostream & os, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4AggregatorAttribute to a file
  //!  descriptor.  Returns the number of bytes written.
  //--------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //               uint32_t Length(uint8_t version = 0) const;               
  //..........................................................................
  //!  Returns the number of bytes needed to store the
  //!  ArtsBgp4AggregatorAttribute on disk.
  //--------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //                       static uint32_t NumObjects()                      
  //..........................................................................
  //!  
  //--------------------------------------------------------------------------
  static uint32_t NumObjects()
  {
    return(_numObjects);
  }
  
private:
  uint16_t     _AS;
  ipv4addr_t   _ipAddr;

  static uint32_t  _numObjects;
};

#endif  // _ARTSBGP4AGGREGATORATTRIBUTE_HH_
