//===========================================================================
// @(#) $Name: arts++-1-1-a6 $
// @(#) $Id: ArtsIpPathData.hh,v 1.11 2000/01/21 08:22:45 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSIPPATHDATA_HH_
#define _ARTSIPPATHDATA_HH_

extern "C" {
#include <sys/time.h>
#include "caida_t.h"
}

#include <vector>

#include "ArtsIpPathEntry.hh"

//----------------------------------------------------------------------------
//     class ArtsIpPathData 
//----------------------------------------------------------------------------
//!  This class encapsulates the data for a forward IP path.  It includes
//!  the IP address of hops and the RTT from the source to the final
//!  destination.
//----------------------------------------------------------------------------
class ArtsIpPathData
{
public:
  //------------------------------------------------------------------------
  //            ArtsIpPathData(ipv4addr_t src, ipv4addr_t dst)
  //........................................................................
  //!  constructor
  //------------------------------------------------------------------------
  ArtsIpPathData(ipv4addr_t src, ipv4addr_t dst);

  //-------------------------------------------------------------------------
  //          ArtsIpPathData(const ArtsIpPathData & artsIpPathData)
  //.........................................................................
  //!  copy constructor
  //-------------------------------------------------------------------------
  ArtsIpPathData(const ArtsIpPathData & artsIpPathData);
    
  //------------------------------------------------------------------------
  //                         ArtsIpPathData()
  //........................................................................
  //!  default constructor
  //------------------------------------------------------------------------
  ArtsIpPathData();

  //------------------------------------------------------------------------
  //                         ~ArtsIpPathData()
  //........................................................................
  //!  destructor
  //------------------------------------------------------------------------
  ~ArtsIpPathData();

  //-------------------------------------------------------------------------
  //             ostream& write(ostream& os, uint8_t version = 0)
  //.........................................................................
  //!  Writes IP path data to an ostream.
  //-------------------------------------------------------------------------
  ostream& write(ostream& os, uint8_t version = 0);

  //-------------------------------------------------------------------------
  //                 int write(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Writes IP path data to a file descriptor.  Returns the number of
  //!  bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //             istream& read(istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads IP path data from an istream.
  //-------------------------------------------------------------------------
  istream& read(istream& is, uint8_t version = 0);

  //-------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //.........................................................................
  //  UNTESTED
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);  // UNTESTED
  
  //-------------------------------------------------------------------------
  //                      inline ipv4addr_t Src() const                      
  //.........................................................................
  //!  Returns the source address of the IP path.
  //-------------------------------------------------------------------------
  inline ipv4addr_t Src() const
  {
    return(this->_src);
  }

  //-------------------------------------------------------------------------
  //                  inline ipv4addr_t Src(ipv4addr_t src)                  
  //.........................................................................
  //!  Sets and returns the source address of the IP path.
  //-------------------------------------------------------------------------
  inline ipv4addr_t Src(ipv4addr_t src)
  {
    this->_src = src;
    return(this->_src);
  }
  
  //-------------------------------------------------------------------------
  //                      inline ipv4addr_t Dst() const                      
  //.........................................................................
  //!  Returns the path destination IP address.
  //-------------------------------------------------------------------------
  inline ipv4addr_t Dst() const
  {
    return(this->_dst);
  }

  //-------------------------------------------------------------------------
  //                  inline ipv4addr_t Dst(ipv4addr_t dst)                  
  //.........................................................................
  //!  Sets and returns the path destination IP address.
  //-------------------------------------------------------------------------
  inline ipv4addr_t Dst(ipv4addr_t dst)
  {
    this->_dst = dst;
    return(this->_dst);
  }

  //-------------------------------------------------------------------------
  //                inline const struct timeval & Rtt() const                
  //.........................................................................
  //!  Returns the round-trip time from the source to the destination and
  //!  back.
  //-------------------------------------------------------------------------
  inline const struct timeval & Rtt() const
  {
    return(this->_rtt);
  }

  //-------------------------------------------------------------------------
  //      inline const struct timeval & Rtt(const struct timeval & rtt)      
  //.........................................................................
  //!  Sets and returns the round-trip time from the source to the
  //!  destination and back.
  //-------------------------------------------------------------------------
  inline const struct timeval & Rtt(const struct timeval & rtt)
  {
    this->_rtt.tv_sec = rtt.tv_sec;
    this->_rtt.tv_usec = rtt.tv_usec;
    return(this->_rtt);
  }

  //-------------------------------------------------------------------------
  //                    inline uint8_t HopDistance() const                   
  //.........................................................................
  //!  Returns the hop distance from the source to the destination.
  //-------------------------------------------------------------------------
  inline uint8_t HopDistance() const
  {
    return(this->_hopDistance);
  }

  //-------------------------------------------------------------------------
  //             inline uint8_t HopDistance(uint8_t hopDistance)             
  //.........................................................................
  //!  Sets and returns the hop distance from the source to the destination.
  //-------------------------------------------------------------------------
  inline uint8_t HopDistance(uint8_t hopDistance)
  {
    this->_hopDistance = hopDistance;
    return(this->_hopDistance);
  }
  
  //-------------------------------------------------------------------------
  //                    inline uint8_t IsComplete() const                    
  //.........................................................................
  //!  Returns 1 if path is complete (destination reached), else returns 0.
  //-------------------------------------------------------------------------
  inline uint8_t IsComplete() const
  {
    return(this->_isComplete);
  }

  //-------------------------------------------------------------------------
  //              inline uint8_t IsComplete(uint8_t isComplete)              
  //.........................................................................
  //!  Sets and returns path complete indicator.
  //-------------------------------------------------------------------------
  inline uint8_t IsComplete(uint8_t isComplete)
  {
    this->_isComplete = isComplete;
    return(this->_isComplete);
  }
  
  //-------------------------------------------------------------------------
  //                      inline uint8_t NumHops() const                     
  //.........................................................................
  //!  Returns the number of hops stored in the path vector.
  //-------------------------------------------------------------------------
  inline uint8_t NumHops() const
  {
    return(this->_numHops);
  }

  //-------------------------------------------------------------------------
  //                 inline uint8_t NumHops(uint8_t numHops)                 
  //.........................................................................
  //!  Sets and returns the number of hops stored in the path vector.
  //-------------------------------------------------------------------------
  inline uint8_t NumHops(uint8_t numHops)
  {
    this->_numHops = numHops;
    return(this->_numHops);
  }
  
  //-------------------------------------------------------------------------
  //              inline vector<ArtsIpPathEntry> & Path() const              
  //.........................................................................
  //!  Returns a reference to the path vector.
  //-------------------------------------------------------------------------
  inline vector<ArtsIpPathEntry> & Path() const
  {
    return(this->_path);
  }

  //-------------------------------------------------------------------------
  //   bool Distinguishable(const vector<ArtsIpPathEntry> & ipPath) const
  //.........................................................................
  //!  Returns true if this->_path is distinguishable from ipPath
  //!  (i.e. for one or more hop numbers, the IP addresses don't match).
  //!  Else returns false.
  //-------------------------------------------------------------------------
  bool Distinguishable(const vector<ArtsIpPathEntry> & ipPath) const;

  //-------------------------------------------------------------------------
  //      bool Distinguishable(const ArtsIpPathData & ipPathData) const
  //.........................................................................
  //!  Compares the hop vectors (using
  //!  this->Distinguishable(ipPathData.Path()), the source IP addresses,
  //!  the destination IP address, and the hop distance (if both paths
  //!  are complete) of this and ipPathData.  If the paths are
  //!  distinguishable by any of these values, we return true, else we
  //!  return false.
  //-------------------------------------------------------------------------
  bool Distinguishable(const ArtsIpPathData & ipPathData) const;
  
  //-------------------------------------------------------------------------
  //    void MergeWithPath(const vector<ArtsIpPathEntry> & ipPath,       
  //                       vector<ArtsIpPathEntry> & mergedPath) const
  //.........................................................................
  //!  Merges this->_path with ipPath, putting the resulting path in
  //!  mergedPath.
  //!  
  //!  In most cases, this should *ONLY* be called if
  //!  this->Distinguishable(ipPath) returns false; the intended use of
  //!  this function is to take paths with missing hops for the same
  //!  src:dst and use them to fill in missing hops and place the
  //!  resulting 'merged' path in mergedPath.  Results are undefined
  //!  if used in any other manner.
  //-------------------------------------------------------------------------
  void MergeWithPath(const vector<ArtsIpPathEntry> & ipPath,
                     vector<ArtsIpPathEntry> & mergedPath) const;

  //-------------------------------------------------------------------------
  //                vector<ipv4addr_t> * HopAddresses() const;               
  //.........................................................................
  //  UNTESTED
  //
  //  Returns a dynamically allocated vector of the IP addresses in
  //  the path.  The caller is responsible for freeing the returned
  //  object.
  //-------------------------------------------------------------------------
  vector<ipv4addr_t> * HopAddresses() const;
  
  //-------------------------------------------------------------------------
  // bool CommonHopAddresses(const vector<ArtsIpPathEntry> & ipPath,       
  //                         vector<ipv4addr_t> & commonHopAddresses) const
  //.........................................................................
  //  UNTESTED
  //
  //  Places common IP addresses from this->_path and ipPath into
  //  commonHopAddresses.  Returns true if this->_path and ipPath had
  //  some matching IP addresses, else returns false.
  //-------------------------------------------------------------------------
  bool CommonHopAddresses(const vector<ArtsIpPathEntry> & ipPath,
                          vector<ipv4addr_t> & commonHopAddresses) const;

  bool CommonHopAddresses(const vector<ipv4addr_t> & hopAddresses,
                          vector<ipv4addr_t> & commonHopAddresses) const;
  
  //-------------------------------------------------------------------------
  //         bool CommonHops(const vector<ArtsIpPathEntry> & ipPath,         
  //                         vector<ArtsIpPathEntry> & commonHops) const
  //.........................................................................
  //  NOT IMPLEMENTED
  //-------------------------------------------------------------------------
  //  bool CommonHops(const vector<ArtsIpPathEntry> & ipPath,
  //                  vector<ArtsIpPathEntry> & commonHops) const;
  
  
  
  //-------------------------------------------------------------------------
  //             void AddHop(ipv4addr_t ipAddr, uint8_t hopNum)
  //.........................................................................
  //!  Appends a hop to the path.  Assigns an IP address of ipAddr and
  //!  a hop number of hopNum to the new hop.   Note we *always* tack the
  //!  new hop on the end of the path, and do *not* use hopNum to determine
  //!  the location.
  //-------------------------------------------------------------------------
  void AddHop(ipv4addr_t ipAddr, uint8_t hopNum);
    
  //-------------------------------------------------------------------------
  //                 uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the number of bytes required to store object contents on disk.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //   ArtsIpPathData & operator = (const ArtsIpPathData & artsIpPathData)
  //.........................................................................
  //!  Overloaded '=' operator for assigning the value of one
  //!  ArtsIpPathData object to another.
  //-------------------------------------------------------------------------
  ArtsIpPathData & operator = (const ArtsIpPathData & artsIpPathData);
  
  //-------------------------------------------------------------------------
  //   friend ostream& operator << (ostream& os,                
  //                                const ArtsIpPathData & artsIpPathData)
  //.........................................................................
  //!  Overloaded ostream '<<' operator to print contents of object.
  //-------------------------------------------------------------------------
  friend ostream& operator << (ostream& os,
                               const ArtsIpPathData & artsIpPathData);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                     static uint32_t NumObjects()
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  ipv4addr_t                        _src;
  ipv4addr_t                        _dst;
  struct timeval                    _rtt;
  uint8_t                           _hopDistance;
  uint8_t                           _isComplete;
  uint8_t                           _numHops;
  mutable vector<ArtsIpPathEntry>   _path;
  static uint32_t                   _numObjects;
};

#endif  /*  _ARTSIPPATHDATA_HH_  */
