//===========================================================================
//  $Name: arts++-1-1-a6 $
//  $Id: ArtsPortMatrixData.hh,v 1.8 2000/01/21 12:21:56 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSPORTMATRIXDATA_HH_
#define _ARTSPORTMATRIXDATA_HH_

extern "C" {
#include "artslocal.h"
#include "caida_t.h"
}

#include <vector>
#ifdef HAVE_IOSTREAM
  #include <iostream>
#else
  #include <iostream.h>
#endif

#include <algorithm>

#include "ArtsPortMatrixEntry.hh"

//---------------------------------------------------------------------------
//  class ArtsPortMatrixData
//---------------------------------------------------------------------------
//!  This class encapsulates the data portion of an ARTS port matrix
//!  object.  A port matrix contains packet and byte counters for traffic
//!  sent from source transport port numbers to destination transport
//!  port numbers.  Like the ArtsPortTableData and
//!  ArtsSelectedPortTableData classes, it can be used to track
//!  traffic by application.  However, keeping data in the
//!  ArtsPortMatrixData format allows you to retain the
//!  sourcePort->destinationPort relationship, which permits more flexible
//!  accounting (and can be used later to generate an
//!  ArtsSelectedPortTable object according to user-specified criteria;
//!  see the ConvertToArtsSelectedPortTable() member of the
//!  ArtsPortMatrixAggregator class, and the ArtsPortChooser class).
//!
//!  This class is not normally instantiated directly, but instead
//!  instantiated from the containing class (an Arts or ArtsPortMatrix
//!  object).
//---------------------------------------------------------------------------
class ArtsPortMatrixData
{
public:
  //-------------------------------------------------------------------------
  //                       inline ArtsPortMatrixData()                       
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsPortMatrixData();

  //--------------------------------------------------------------------------
  //                          ~ArtsPortMatrixData()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsPortMatrixData();

  //-------------------------------------------------------------------------
  //                  inline uint16_t SampleInterval() const
  //.........................................................................
  //!  Returns the sampling ratio if the port matrix was built using
  //!  packet sampling techniques.  Normally this value is 1, meaning
  //!  we're working with complete data sets (for example, from Cisco
  //!  flow-export).
  //-------------------------------------------------------------------------
  inline uint16_t SampleInterval() const
  {
    return(this->_sampleInterval);
  }

  //-------------------------------------------------------------------------
  //         inline uint16_t SampleInterval(uint16_t sampleInterval)         
  //.........................................................................
  //!  Sets and returns the sampling ratio if the port matrix was built
  //!  using packet sampling techniques.  Normally this value is 1,
  //!  meaning we're working with complete data sets (for example, from
  //!  Cisco flow-export).
  //-------------------------------------------------------------------------
  inline uint16_t SampleInterval(uint16_t sampleInterval)
  {
    this->_sampleInterval = sampleInterval;
    return(this->_sampleInterval);
  }
  
  //-------------------------------------------------------------------------
  //                      inline uint32_t Count() const
  //.........................................................................
  //!  Returns the number of port matrix entries (cells) in an
  //!  ArtsPortMatrixData object.
  //-------------------------------------------------------------------------
  inline uint32_t Count() const
  {
    return(this->_count);
  }

  //-------------------------------------------------------------------------
  //                  inline uint32_t Count(uint32_t count)                  
  //.........................................................................
  //!  Sets and Returns the number of port matrix entries (cells) in an
  //!  ArtsPortMatrixData object.
  //-------------------------------------------------------------------------
  inline uint32_t Count(uint32_t count)
  {
    this->_count = count;
    return(this->_count);
  }
  
  //-------------------------------------------------------------------------
  //                    inline uint64_t TotalPkts() const
  //.........................................................................
  //!  Returns the total packets of traffic in the ArtsPortMatrixData
  //!  object.  The total is the sum of all traffic from each source to
  //!  each destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalPkts() const
  {
    return(this->_totpkts);
  }

  //-------------------------------------------------------------------------
  //              inline uint64_t TotalPkts(uint64_t totalPkts)              
  //.........................................................................
  //!  Sets and returns the total packets of traffic in the
  //!  ArtsPortMatrixData object.  The total is the sum of all traffic
  //!  from each source to each destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalPkts(uint64_t totalPkts)
  {
    this->_totpkts = totalPkts;
    return(this->_totpkts);
  }
  
  //-------------------------------------------------------------------------
  //                    inline uint64_t TotalBytes() const
  //.........................................................................
  //!  Returns the total bytes of traffic in the ArtsPortMatrixData
  //!  object.  The total is the sum of all traffic from each source to
  //!  each destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalBytes() const
  {
    return(this->_totbytes);
  }

  //-------------------------------------------------------------------------
  //             inline uint64_t TotalBytes(uint64_t totalBytes)             
  //.........................................................................
  //!  Sets and returns the total bytes of traffic in the
  //!  ArtsPortMatrixData object.  The total is the sum of all traffic
  //!  from each source to each destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalBytes(uint64_t totalBytes)
  {
    this->_totbytes = totalBytes;
    return(this->_totbytes);
  }
  
  //-------------------------------------------------------------------------
  //                     inline uint64_t Orphans() const                     
  //.........................................................................
  //                                                                         
  //-------------------------------------------------------------------------
  inline uint64_t Orphans() const
  {
    return(this->_orphans);
  }

  //-------------------------------------------------------------------------
  //                inline uint64_t Orphans(uint64_t orphans)                
  //.........................................................................
  //                                                                         
  //-------------------------------------------------------------------------
  inline uint64_t Orphans(uint64_t orphans)
  {
    this->_orphans = orphans;
    return(this->_orphans);
  }
  
  //-------------------------------------------------------------------------
  //          inline vector<ArtsPortMatrixEntry> & PortEntries() const
  //.........................................................................
  //!  Return a reference to the vector of ArtsPortMatrixEntry objects.
  //!  Note that even though this member may be called from a const
  //!  object, the returned reference is not const; the vector is
  //!  mutable.
  //-------------------------------------------------------------------------
  inline vector<ArtsPortMatrixEntry> & PortEntries() const
  {
    return(this->_portEntries);
  }

  //-------------------------------------------------------------------------
  //              uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the length required to store the ArtsPortMatrixData object
  //!  on disk.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //            istream& read(istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsPortMatrixData object from an istream.  Returns the
  //!  istream.
  //-------------------------------------------------------------------------
  istream& read(istream& is, uint8_t version = 0);

  //-------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsPortMatrixData object from a file descriptor.
  //!  Returns the number of bytes read if successful, -1 on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
    
  //-------------------------------------------------------------------------
  //             ostream& write(ostream& os, uint8_t version = 0)
  //.........................................................................
  //!  Writes an ArtsPortMatrixData object to an ostream.  Returns the
  //!  ostream.
  //-------------------------------------------------------------------------
  ostream& write(ostream& os, uint8_t version = 0);

  //-------------------------------------------------------------------------
  //                  int write(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Writes an ArtsPortMatrixData object to a file descriptor.  Returns
  //!  the number of bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //    ostream& operator << (ostream& os,                   
  //                          const ArtsPortMatrixData & artsPortMatrixData)
  //.........................................................................
  //!  Overloaded '<<' operator.  Dumps the contents of an
  //!  ArtsPortMatrixData object to an ostream in a human-readable form.
  //!  Returns the ostream.
  //-------------------------------------------------------------------------
  friend ostream& operator << (ostream& os,
                               const ArtsPortMatrixData & artsPortMatrixData);
  
  //-------------------------------------------------------------------------
  //                        void SortEntriesByBytes()
  //.........................................................................
  //!  Sorts the port matrix entries in desending order by byte count
  //!  (the entry with the most traffic in bytes will become the first
  //!  entry).  Note this uses the ArtsPortMatrixEntryGreaterBytes
  //!  function object defined in ArtsPortMatrixEntry.hh
  //-------------------------------------------------------------------------
  void SortEntriesByBytes();
  
  //-------------------------------------------------------------------------
  //                        void SortEntriesByPkts()
  //.........................................................................
  //!  Sorts the port matrix entries in descending order by packet count
  //!  (the entry with the most traffic in packets will become the first
  //!  entry).  Note this uses the ArtsPortMatrixEntryGreaterPkts
  //!  function object defined in ArtsPortMatrixEntry.hh
  //-------------------------------------------------------------------------
  void SortEntriesByPkts();

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                      
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG
  
private:
  uint16_t                             _sampleInterval;
  uint32_t                             _count;
  uint64_t                             _totpkts;
  uint64_t                             _totbytes;
  uint64_t                             _orphans;
  mutable vector<ArtsPortMatrixEntry>  _portEntries;

  static uint32_t                      _numObjects;
};

#endif  // _ARTSPORTMATRIXDATA_HH_
