//===========================================================================
//  @(#) $Name: arts++-1-1-a6 $
//  @(#) $Id: ArtsProtocolTableAggregator.hh,v 1.3 2000/01/21 12:21:57 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSPROTOCOLTABLEAGGREGATOR_HH_
#define _ARTSPROTOCOLTABLEAGGREGATOR_HH_

#include <map>
#include <vector>

#include "ArtsProtocolTable.hh"

//---------------------------------------------------------------------------
//  class ArtsProtocolTableAggregator
//---------------------------------------------------------------------------
//!  This class is used to aggregate protocol table objects in the time
//!  domain.
//---------------------------------------------------------------------------
class ArtsProtocolTableAggregator
{
public:
  //--------------------------------------------------------------------------
  //!  Holds a packet and byte counter.
  //--------------------------------------------------------------------------
  typedef struct {
    uint64_t  Pkts;
    uint64_t  Bytes;
  } counter_t;
  
  //-------------------------------------------------------------------------
  //               ArtsProtocolTableAggregator(const Arts & arts)
  //.........................................................................
  //!  Construct using an Arts object.  \c arts must be a protocal table
  //!  object.
  //-------------------------------------------------------------------------
  ArtsProtocolTableAggregator(const Arts & arts);
  
  //-------------------------------------------------------------------------
  //                       void Add(const Arts & arts)
  //.........................................................................
  //!  Adds the contents of an Arts object.  The input Arts object must be
  //!  a protocol table object.  The effect here is to add any new protocol
  //!  table entries and add to the counters for existing net matrix
  //!  entries.
  //-------------------------------------------------------------------------
  void Add(const Arts & arts);
  
  //-------------------------------------------------------------------------
  //              ArtsProtocolTable *ConvertToArtsProtocolTable() const
  //.........................................................................
  //!  Return an ArtsProtocolTable object (dynamically allocated)
  //!  containing the aggregate data.  The caller is responsible for
  //!  calling delete() on the returned ArtsProtocolTable pointer.
  //-------------------------------------------------------------------------
  ArtsProtocolTable *ConvertToArtsProtocolTable() const;
  
private:
  ArtsHeader                               _header;
  vector<ArtsAttribute>                    _attributes;
  map<uint8_t,counter_t,less<uint8_t> >    _protocolCounters;
  
};

#endif  // _ARTSPROTOCOLTABLEAGGREGATOR_HH_
