//===========================================================================
// @(#) $Name: arts++-1-1-a6 $
// @(#) $Id: ArtsTosTableAggregatorMap.hh,v 1.1 1999/04/30 19:54:59 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSTOSTABLEAGGREGATORMAP_HH_
#define _ARTSTOSTABLEAGGREGATORMAP_HH_

#include <map>

#include "ArtsTosTableAggregator.hh"
#include "ArtsAggregatorMapKey.hh"

typedef map<ArtsAggregatorMapKey,ArtsTosTableAggregator *,less<ArtsAggregatorMapKey> >  _artsTosTableAggregatorMap_t_;

//---------------------------------------------------------------------------
//  class ArtsTosTableAggregatorMap :
//    public _artsTosTableAggregatorMap_t_
//---------------------------------------------------------------------------
//  This class abstracts a collection of ArtsTosTableAggregator
//  objects keyed by an ArtsAggregatorMapKey object.  It can be used to
//  build multiple ArtsTosTableAggregator objects from
//  ArtsTosTable objects.
//
//  The ArtsAggregatorMapKey class contains a router IP address
//  and an interface index.  These are the keys we need to keep data
//  separate per router interface, and should be present in the input
//  data (as host and interface attributes in Arts::Attributes()).
//
//  Once you're done loading an ArtsTosTableAggregatorMap using
//  Add() one or more times, you can iterate over the entries and do
//  whatever you'd normally do with an ArtsTosTableAggregator
//  object (typically, convert to an ArtsTosTable object and write
//  to an ofstream).  Note that since an STL <map> iterator dereference
//  returns a <pair>, you'll have access to the key (an
//  ArtsAggregatorMapKey) via (*iter).first and the value (an
//  ArtsTosTableAggregator) via (*iter).second.  Standard STL
//  stuff...
//
//  Note we just inherit from an STL map instantiation, since we're not
//  much more than a pure <map>.
//---------------------------------------------------------------------------
class ArtsTosTableAggregatorMap :
  public _artsTosTableAggregatorMap_t_
{
public:
  //-------------------------------------------------------------------------
  //  Some typedefs to make things obvious.
  //-------------------------------------------------------------------------
  typedef _artsTosTableAggregatorMap_t_::iterator        iterator;
  typedef _artsTosTableAggregatorMap_t_::const_iterator  const_iterator;

  ~ArtsTosTableAggregatorMap()
  {
    iterator  iter;

    for (iter = (*this).begin(); iter != (*this).end(); iter++)
      delete((*iter).second);
    (*this).erase((*this).begin(),(*this).end());
  }
  
  //-------------------------------------------------------------------------
  //                       int Add(const Arts & arts)
  //.........................................................................
  //  Adds the data contained in an Arts object (which should be a
  //  tos table object) to the aggregator map.  We use the
  //  Attributes() in the input Arts object to determine where in the
  //  map we should add the data (router and interface index).  Returns
  //  0 on success, -1 on failure (i.e. if the Arts object does not
  //  contain tos table data).
  //-------------------------------------------------------------------------
  int Add(const Arts & arts);

  //-------------------------------------------------------------------------
  //     int AddAllFromFile(const string & fileName, bool quiet = true)
  //.........................................................................
  //  Adds the data contained in all tos table objects in the file
  //  named by fileName.  Returns the number of tos table objects
  //  added on success, -1 on failure.  If quiet is set to false, this
  //  function will print a single '.'  on stdout every time it reads a
  //  tos table object.
  //-------------------------------------------------------------------------
  int AddAllFromFile(const string & fileName, bool quiet = true);
  
  
};

#endif  // _ARTSTOSTABLEAGGREGATORMAP_HH_
