//===========================================================================
//  @(#) $Name: arts++-1-1-a6 $
//  @(#) $Id: ArtsTosTableData.hh,v 1.3 2000/01/21 12:21:58 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSTOSTABLEDATA_HH_
#define _ARTSTOSTABLEDATA_HH_

extern "C" 
{
#include <sys/types.h>

#include "caida_t.h"
}

#include <vector>

#include "ArtsTosTableEntry.hh"

//---------------------------------------------------------------------------
//  class ArtsTosTableData
//---------------------------------------------------------------------------
//!  This class encapsulates the data portion of an ARTS TOS table
//!  object.  It contains a vector of TOS entries, each of which
//!  contains the TOS value plus packet and byte counters for traffic
//!  using the TOS.
//---------------------------------------------------------------------------
class ArtsTosTableData
{
public:
  //-------------------------------------------------------------------------
  //                           ArtsTosTableData()
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsTosTableData();
  
  //-------------------------------------------------------------------------
  //                          ~ArtsTosTableData()
  //.........................................................................
  //!  destructor
  //-------------------------------------------------------------------------
  ~ArtsTosTableData();
  
  //-------------------------------------------------------------------------
  //                     uint16_t SampleInterval() const
  //.........................................................................
  //!  Returns the sampling ratio, indicating the 1:N packet sampling
  //!  used to obtain the data.  Normally this value is 1, meaning we're
  //!  working with complete data sets (for example, from Cisco
  //!  flow-export).
  //-------------------------------------------------------------------------
  uint16_t SampleInterval() const;
  
  //-------------------------------------------------------------------------
  //            uint16_t SampleInterval(uint16_t sampleInterval)
  //.........................................................................
  //!  Sets and returns the sampling ratio, indicating the 1:N packet
  //!  sampling used to obtain the data.  Normally this value is 1,
  //!  meaning we're working with complete data sets (for example, from
  //!  Cisco flow-export).
  //-------------------------------------------------------------------------
  uint16_t SampleInterval(uint16_t sampleInterval);
  
  //-------------------------------------------------------------------------
  //                       uint64_t TotalPkts() const
  //.........................................................................
  //!  Returns the total packets of traffic in the ArtsTosTableData
  //!  object.  The total is the sum of all traffic for all TOS values
  //!  contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalPkts() const;
  
  //-------------------------------------------------------------------------
  //                 uint64_t TotalPkts(uint64_t totalPkts)
  //.........................................................................
  //!  Sets and returns the total packets of traffic in the
  //!  ArtsTosTableData object.  The total is the sum of all traffic
  //!  for all TOS values contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalPkts(uint64_t totalPkts);
  
  //-------------------------------------------------------------------------
  //                       uint64_t TotalBytes() const
  //.........................................................................
  //!  Returns the total bytes of traffic in the ArtsTosTableData
  //!  object.  The total is the sum of all traffic for all TOS values
  //!  contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalBytes() const;
  
  //-------------------------------------------------------------------------
  //                uint64_t TotalBytes(uint64_t totalBytes)
  //.........................................................................
  //!  Sets and returns the total bytes of traffic in the
  //!  ArtsTosTableData object.  The total is the sum of all traffic
  //!  for all TOS values contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalBytes(uint64_t totalBytes);
  
  //-------------------------------------------------------------------------
  //            vector<ArtsTosTableEntry> & TosEntries() const
  //.........................................................................
  //!  Returns a reference to the vector of TOS entries in the
  //!  ArtsTosTableData object.
  //-------------------------------------------------------------------------
  vector<ArtsTosTableEntry> & TosEntries() const;

  //-------------------------------------------------------------------------
  //                        void SortEntriesByBytes()
  //.........................................................................
  //  
  //-------------------------------------------------------------------------
  void SortEntriesByBytes();

  void SortEntriesByPkts();

  //-------------------------------------------------------------------------
  //            uint32_t ComputeLength(uint8_t version = 0) const
  //.........................................................................
  //!  Computes the bytes required to store the ArtsTosTableData
  //!  object in a file.
  //-------------------------------------------------------------------------
  uint32_t ComputeLength(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //                uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the bytes required to store the ArtsTosTableData
  //!  object in a file.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //           istream& read(istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads the contents of an ArtsTosTableData from an istream.
  //!  Returns the istream.
  //-------------------------------------------------------------------------
  istream& read(istream& is, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //                 int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads the contents of an ArtsTosTableData from a file
  //!  descriptor.  Returns the number of bytes read if successful, -1
  //!  on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //          ostream& write(ostream& os, uint8_t version = 0) const
  //.........................................................................
  //!  Writes the contents of an ArtsTosTableData to an ostream.
  //!  Returns the ostream.
  //-------------------------------------------------------------------------
  ostream& write(ostream& os, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const
  //.........................................................................
  //!  Writes the contents of an ArtsTosTableData to a file
  //!  descriptor.  Returns the number of bytes written on success, -1
  //!  on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //    ArtsTosTableData &
  //    operator = (const ArtsTosTableData & artsTosTableData)
  //.........................................................................
  //!  Overloaded '=' operator for assigning one ArtsTosTableData
  //!  to another.
  //-------------------------------------------------------------------------
  ArtsTosTableData &
  operator = (const ArtsTosTableData & artsTosTableData);

  //-------------------------------------------------------------------------
  //    friend ostream &                            
  //    operator << (ostream& os,                   
  //                 const ArtsTosTableData & artsTosTableData)
  //.........................................................................
  //!  Overload ostream '<<' operator to dump the contents of an
  //!  ArtsTosTableData object to an ostream in a human-readable
  //!  form.  Returns the ostream.
  //-------------------------------------------------------------------------
  friend ostream &
  operator << (ostream& os,
               const ArtsTosTableData & artsTosTableData);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                      
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  uint16_t                            _sampleInterval;
  uint64_t                            _totpkts;
  uint64_t                            _totbytes;
  mutable vector<ArtsTosTableEntry>   _tosEntries;
  mutable uint32_t                    _length;

  static uint32_t                     _numObjects;
};

#endif  // _ARTSTOSTABLEDATA_HH_
