//===========================================================================
//  $Name: arts++-1-1-a6 $
//  $Id: ArtsTosTableEntry.hh,v 1.4 2000/01/21 12:21:58 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSTOSTABLEENTRY_HH_
#define _ARTSTOSTABLEENTRY_HH_

extern "C" {
#include "artslocal.h"
#include <sys/types.h>
#include "caida_t.h"
}

#ifdef HAVE_IOSTREAM
  #include <iostream>
#else
  #include <iostream.h>
#endif

#ifdef HAVE_IOMANIP
  #include <iomanip>
#else
  #include <iomanip.h>
#endif

//---------------------------------------------------------------------------
//  class ArtsTosTableEntry
//---------------------------------------------------------------------------
//!  Class representing a TOS table entry.  This class is used to hold
//!  IP traffic counters (number of packets and bytes) per TOS (Type Of
//!  Service) value.
//---------------------------------------------------------------------------
class ArtsTosTableEntry
{
public:
  //-------------------------------------------------------------------------
  //                       ArtsTosTableEntry()
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsTosTableEntry();
  
  //-------------------------------------------------------------------------
  //                       ~ArtsTosTableEntry()
  //.........................................................................
  //!  destructor
  //-------------------------------------------------------------------------
  ~ArtsTosTableEntry();

  //-------------------------------------------------------------------------
  //                    uint8_t TosNumber() const
  //.........................................................................
  //!  Returns the TOS value for the TOS entry.
  //-------------------------------------------------------------------------
  uint8_t TosNumber() const;
  
  //-------------------------------------------------------------------------
  //                uint8_t TosNumber(uint8_t tos)
  //.........................................................................
  //!  Sets and returns the TOS number for the TOS entry.
  //-------------------------------------------------------------------------
  uint8_t TosNumber(uint8_t tos);
  
  //-------------------------------------------------------------------------
  //                       uint64_t Pkts() const
  //.........................................................................
  //!  Returns the packet count for the TOS entry.
  //-------------------------------------------------------------------------
  uint64_t Pkts() const;
  
  //-------------------------------------------------------------------------
  //                    uint64_t Pkts(uint64_t pkts)
  //.........................................................................
  //!  Sets and returns the packet count for the TOS entry.
  //-------------------------------------------------------------------------
  uint64_t Pkts(uint64_t pkts);
  
  //-------------------------------------------------------------------------
  //                      inline uint64_t Bytes() const
  //.........................................................................
  //!  Returns the byte count for the TOS entry.
  //-------------------------------------------------------------------------
  uint64_t Bytes() const;
  
  //-------------------------------------------------------------------------
  //                  inline uint64_t Bytes(uint64_t bytes)
  //.........................................................................
  //!  Sets and returns the byte count for the TOS entry.
  //-------------------------------------------------------------------------
  uint64_t Bytes(uint64_t bytes);

  //-------------------------------------------------------------------------
  //                uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the number of bytes needed to store the TOS entry in a
  //!  file.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //                       uint8_t Descriptor() const
  //.........................................................................
  //!  Returns the descriptor field.
  //-------------------------------------------------------------------------
  uint8_t Descriptor() const;

  //-------------------------------------------------------------------------
  //                 uint8_t Descriptor(uint8_t descriptor)
  //.........................................................................
  //!  Sets and returns the descriptor field.
  //-------------------------------------------------------------------------
  uint8_t Descriptor(uint8_t descriptor);

  //-------------------------------------------------------------------------
  //            istream& read(istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsTosTableEntry from an istream.  Returns the istream.
  //-------------------------------------------------------------------------
  istream& read(istream& is, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //                 int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsTosTableEntry from a file descriptor.  Returns the
  //!  number of bytes read on success, -1 on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //          ostream& write(ostream& os, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsTosTableEntry to an ostream.  Returns the ostream.
  //-------------------------------------------------------------------------
  ostream& write(ostream& os, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //               int write(int fd, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsTosTableEntry to a file descriptor.  Returns the
  //!  number of bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //   ArtsTosTableEntry &                        
  //   operator = (const ArtsTosTableEntry & artsTosTableEntry)
  //.........................................................................
  //!  Overloaded '=' operator to assign one ArtsTosTableEntry to
  //!  another.
  //-------------------------------------------------------------------------
  ArtsTosTableEntry &
  operator = (const ArtsTosTableEntry & artsTosTableEntry);

  //-------------------------------------------------------------------------
  //   friend ostream &                            
  //   operator << (ostream& os,                   
  //                const ArtsTosTableEntry & artsTosTableEntry)
  //.........................................................................
  //!  Overloaded '<<' operator to dump the contents of an
  //!  ArtsTosTableEntry to an ostream in a human-readable form.
  //!  Returns the ostream.
  //-------------------------------------------------------------------------
  friend ostream &
  operator << (ostream& os,
               const ArtsTosTableEntry & artsTosTableEntry);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                      
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  uint8_t     _tosNum;
  uint8_t     _descriptor;
  uint64_t    _pkts;
  uint64_t    _bytes;

  static uint32_t  _numObjects;
};

//---------------------------------------------------------------------------
//  class ArtsTosEntryGreaterBytes
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting
//!  ArtsTosTableEntry objects in descending order by bytes of
//!  traffic.
//---------------------------------------------------------------------------
class ArtsTosEntryGreaterBytes
{
public:
  bool operator()(const ArtsTosTableEntry & tosEntry1,
                  const ArtsTosTableEntry & tosEntry2) const;
};

//---------------------------------------------------------------------------
//  class ArtsTosEntryGreaterPkts
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting
//!  ArtsTosTableEntry objects in descending order by packets of
//!  traffic.
//---------------------------------------------------------------------------
class ArtsTosEntryGreaterPkts
{
public:
  bool operator()(const ArtsTosTableEntry & tosEntry1,
                  const ArtsTosTableEntry & tosEntry2) const;
};


#endif  // _ARTSTOSTABLEENTRY_HH_
